from cgi import parse_qs
from lxml.etree import parse
from meresco.components.http import utils as httputils
from meresco.framework import Observable
from urlparse import urlsplit
from xml.sax.saxutils import escape
from bzv.jsonencoder import JSONEncoder

namespaceMap = {
    'didl' :"urn:mpeg:mpeg21:2002:02-DIDL-NS",
    'mods' :"http://www.loc.gov/mods/v3",
    'rdf'  :"http://www.w3.org/1999/02/22-rdf-syntax-ns#",
    'extra':"http://meresco.com/namespace/fields/extra",
    'meta' :"http://meresco.com/namespace/harvester/meta",
}

def okJSON():
    CRLF = "\r\n"
    ContentTypeJSON = "Content-Type: application/json"
    Ok = "HTTP/1.0 200 Ok" + CRLF
    okJSON = "HTTP/1.0 200 OK" + CRLF + \
             ContentTypeJSON + CRLF + \
             CRLF
    return okJSON

def xpath(node, path):
    result = node.xpath(path, namespaceMap)
    return result and result[0] or ''

def nodeset(node, path):
    return node.xpath(path, namespaceMap)

def add(aDictionary, key, value):
    if value:
            aDictionary[key] = value

def makeUrlContent(url):
    u = '<img src="http://dbiref.uvt.nl/images/edoc.gif" alt="*" border="0"/> <a target="_blank" href="%(url)s">%(url)s</a>' %{'url': escape(url)}
    return u

class Record(Observable):
    def __init__(self):
         Observable.__init__(self)

    def setMetadata(self, recordId):
       if recordId.split(':')[0] in ['opc-uvt-nl']:
           self._metadata = parse(self.any.getStream(recordId, 'metadata'))
       else:
           self._metadata = parse(self.any.getStream(recordId, 'metadata'))

    def getMetadata(self, recordId):
       return self._metadata

    def getOrder(self):
###### Note that all fields must be listed here
######      otherwise they will not be displayed.
###### This is a public method.
       order = {\
           1: 'Title',\
           2: 'Language',\
           3: 'Author(s)',\
           4: 'Editor(s)',\
           5: 'Supervisor(s)',\
           6: 'Corporation',\
           7: 'Source',\
           8: 'Publication date',\
           9: 'URL(s)',\
           10: 'Abstract',\
           11: 'Keyword(s)',\
           12: 'Classification',\
           13: 'ISSN',\
           14: 'ISBN',\
           15: 'Publisher',\
           16: 'Extent',\
           17: 'Note(s)',\
           18: 'Reference',\
           19: 'Publication type',\
           20: 'Location'
       }
       return order

    def createRecord(self, recordId):
       self.setMetadata(recordId)
       self.record = {}
       add(self.record, 'Title', self._getTitle())
       add(self.record, 'Language', self._getLanguage())
       self._handleNames()
       add(self.record, 'Source', self._getSource())
       add(self.record, 'Publication date', self._getDate())
       add(self.record, 'URL(s)', self._getUrls())
       add(self.record, 'Abstract', self._getAbstract())
       add(self.record, 'Keyword(s)', self._getKeywords())
       add(self.record, 'Classification', self._getClassification())
       self._handleIdentifiers()
       add(self.record, 'Publisher', self._getPublisher())
       add(self.record, 'Publication type', self._getGenre())
       add(self.record, 'Extent', self._getExtent())
       add(self.record, 'Note(s)', self._getNotes())
       add(self.record, 'Reference', self._getReference())
       add(self.record, 'Location', self._getLocation())
       return self.record

    def getMods(self, recordId):
        self.setMetadata(recordId)
        return xpath(self._metadata, '//mods:mods')

    def _addName(self, names, name):
        family = ""
        given  = ""
        for namePart in nodeset(name, 'mods:namePart'):
            type = xpath(namePart, '@type')
            if type == 'family' or not type:
               family = xpath(namePart, 'text()')
            elif type == 'given':  given  = xpath(namePart, 'text()')
        if family: names += family
        if given:  names  += ', ' + given
        if family: names += '; '
        return names

    def _getDate(self):
        date = xpath(self._metadata, '//mods:originInfo/mods:dateIssued/text()')
        if date:
            altDate = date.split('(', 1)
            if len(altDate) == 2: return altDate[1].rstrip(')')
            else: return date

    def _getSource(self):
        editors = self._getHostEditors()
	title = self._getHostTitle()
        if not title: return ''
	if editors: return editors + ' ' + '<i>' + title + '</i>'
        date = self._getDate()
        volume = xpath(self._metadata, '//mods:part/mods:detail[@type="volume"]/mods:number/text()')
        issue = xpath(self._metadata, '//mods:part/mods:detail[@type="issue"]/mods:number/text()')
        firstpage = xpath(self._metadata, '//mods:part/mods:extent/mods:start/text()')
        endpage = xpath(self._metadata, '//mods:part/mods:extent/mods:end/text()')
        if volume: title += ' vol. ' + volume
        if date: title += ' (' + date + ')'
        if issue: title += ' nr. ' + issue
        if firstpage:
            title += ' p.' + firstpage
            if endpage: title += '-' + endpage
	part = xpath(self._metadata, '//mods:relatedItem[@type="host"]/mods:part/mods:text/text()')
        if part: title += '. ' + part
	return title

    def _getTitle(self):
        title = xpath(self._metadata, '//mods:mods/mods:titleInfo/mods:title/text()')
        if not title: 
            title = xpath(self._metadata, '//mods:relatedItem[@type="host"]/mods:titleInfo/mods:title/text()')
            if title: title = '[' + title + ']'
            else: title = '[NO TITLE AVAILABLE]'
        subTitles = nodeset(self._metadata, '//mods:mods/mods:titleInfo/mods:subTitle/text()')
        if subTitles:
            for subTitle in subTitles:
                title += ": " + subTitle
        return escape(title)

    def _handleNames(self):
        authors = ""
        editors = ""
        supervisors = ""
	names = (nodeset(self._metadata, '//mods:mods/mods:name'))
	if not names: return
        for name in names:
            role = xpath(name, 'mods:role/mods:roleTerm/text()')
            if   role == 'aut' or not role:
                authors     = self._addName(authors,     name)
            elif role == 'edt':
                editors     = self._addName(editors,     name)
            elif role == 'ths':
                supervisors = self._addName(supervisors, name)
            elif role == 'dgg':
                content = 'Degree granted by ' + str(xpath(name, 'mods:namePart/text()'))
                add(self.record, 'Corporation',   content)
            if authors:
                add(self.record, 'Author(s)',     authors.rstrip(' ;'))
            if editors:
                add(self.record, 'Editor(s)',     editors.rstrip(' ;'))
            if supervisors:
                add(self.record, 'Supervisor(s)', supervisors.rstrip(' ;'))

    def _getExtent(self):
        return xpath(self._metadata, '//mods:mods/mods:physicalDescription/mods:extent/text()')

    def _getLanguage(self):
        return xpath(self._metadata, '//mods:mods/mods:language/mods:languageTerm/text()')

    def _getGenre(self):
        genre = xpath(self._metadata, '//mods:mods/mods:genre/text()')
        if genre and '/' in genre: genre = genre.split('/')[-1]
	return genre

    def _getHostTitle(self):
        return xpath(self._metadata, '//mods:mods/mods:relatedItem[@type="host"]/mods:titleInfo/mods:title/text()')

    def _getHostEditors(self):
        editors = ""
	family = ""
	given = ""
        for name in nodeset(self._metadata, '//mods:mods/mods:relatedItem/mods:name'):
            if xpath(name, 'mods:role/mods:roleTerm/text()') == 'edt':
                for namePart in nodeset(name, 'mods:namePart'):
                    type = xpath(namePart, '@type')
                    if type == 'family': family = xpath(namePart, 'text()')
                    elif type == 'given':  given = xpath(namePart, 'text()')
                if family: editors += family
                if given:  editors += ', ' + given
		if family: editors += '; '
		family = ""
		given = ""
        if editors: return editors.rstrip('; ') + ' (Ed.)'
	else: return ''
        
    def _getAbstract(self):
        return escape(xpath(self._metadata, '//mods:abstract/text()'))

    def _getUrls(self):
        urls = []
        url = xpath(self._metadata, '//mods:location/mods:url/text()')
        if url: urls.append(url)
        for Item in nodeset(self._metadata, '//didl:DIDL/didl:Item/didl:Item'):
             if xpath(Item, 'didl:Descriptor/didl:Statement/rdf:type/text()') == 'info:eu-repo/semantics/objectFile':
                 urls.append(xpath(Item, 'didl:Component/didl:Resource/@ref'))
        return urls

    def _getNotes(self):
        notes = ""
        count = 0
        for note in nodeset(self._metadata, '//mods:mods/mods:note/text()'):
             count += 1
             if count > 1:
                 notes += ". "
             notes += note
        return notes

    def _getReference(self):
        return xpath(self._metadata, '//mods:mods/mods:relatedItem[@type="isReferencedBy"]/mods:titleInfo/mods:title/text()')

    def _getPublisher(self):
        place = xpath(self._metadata, '//mods:mods/mods:originInfo/mods:place/mods:placeTerm/text()')
        name = xpath(self._metadata, '//mods:mods/mods:originInfo/mods:publisher/text()')
        publisher = ""
        if name:
           publisher = ""
           if place: publisher = place + ": "
           publisher += name
        return publisher

    def _getKeywords(self):
        keywords = ""
        for topic in nodeset(self._metadata, '//mods:mods/mods:subject/mods:topic'):
            keywords += xpath(topic, 'text()') + '; '
	if keywords: return keywords.rstrip('; ')
	else: return ''

    def _getClassification(self):
        string = ""
	for classification in nodeset(self._metadata, '//mods:mods/mods:classification'):
            string += xpath(classification, 'text()') + '; '
	return string.rstrip('; ')

    def _getLocation(self):
        return xpath(self._metadata, '//mods:mods/mods:location/mods:holdingSimple/mods:copyInformation/mods:shelfLocator/text()')

    def _handleIdentifiers(self):
	identifiers = (nodeset(self._metadata, '//mods:identifier/text()'))
	if not identifiers: return
	for id in identifiers:
            idParts = id.split(':')
            ### urn:{issn|issbn}:<value>
            if len(idParts) < 3: break
            if idParts[1] == 'issn' or idParts[1] == 'ISSN':
                add(self.record, 'ISSN', idParts[2])
            if idParts[1] == 'isbn' or idParts == 'ISBN':
                add(self.record, 'ISBN', idParts[2])

class RecordXml(Observable):
    def __init__(self):
         Observable.__init__(self)

    def handleRequest(self, RequestURI='', **kwargs):
####### TP: toevoegen het afvangen van exceptions: 1. er is geen record id, 
####### 2. er wordt geen metadata gevonden
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        arguments = parse_qs(Query)
        recordId = arguments.get('recordId', [''])[0]
        record = self.any.createRecord(recordId)
        yield httputils.okXml
        yield """<?xml version="1.0" encoding="UTF-8"?>
<record>"""
        
####### TP: labels zijn in het Engels. Toevoegen: ondersteunen van andere talen.
####### TP: ook handig is om per field het type toe te voegen (taalonafhankelijk)
        order = self.any.getOrder()
        order.keys().sort()
        for key in order.keys():
            label = order[key]
            if label in record:
                if label == 'URL(s)':
                    for url in record['URL(s)']:
                        yield '\n <field><label>URL</label><content>%s</content></field>' %(makeUrlContent(url))
                else:
                    yield  '\n <field><label>%s</label><content>%s</content></field>' %(label, record[label])
        yield '\n</record>'

class RecordJSON(Observable):
    def __init__(self):
         Observable.__init__(self)

    def handleRequest(self, RequestURI='', **kwargs):
####### TP: toevoegen het afvangen van exceptions: 1. er is geen record id, 
####### 2. er wordt geen metadata gevonden
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        arguments = parse_qs(Query)
        recordId = arguments.get('recordId', [''])[0]
        record = self.any.createRecord(recordId)
        yield okJSON()
        yield JSONEncoder().encode(record)
